@@ -5,9 +5,9 @@
 ###
 ### Plot-method(s) for fitted hhh4() models
 ###
-### Copyright (C) 2010-2012 Michaela Paul, 2012-2015 Sebastian Meyer
-### $Revision: 1504 $
-### $Date: 2015-10-23 16:52:26 +0200 (Fre, 23. Okt 2015) $
+### Copyright (C) 2010-2012 Michaela Paul, 2012-2018 Sebastian Meyer
+### $Revision: 2070 $
+### $Date: 2018-01-22 17:50:23 +0100 (Mon, 22. Jan 2018) $
 ################################################################################
 
 
@@ -38,20 +38,35 @@ plotHHH4_fitted <- function (x, units = 1, names = NULL,
                              par.settings = list(),
                              legend = TRUE, legend.args = list(),
                              legend.observed = FALSE,
-                             decompose = NULL, meanHHH = NULL, ...)
+                             decompose = NULL, total = FALSE, meanHHH = NULL, ...)
 {
-    if (is.null(units)) units <- seq_len(x$nUnit)
+    if (total) {
+        units <- "Overall"  # only used as a label
+    } else if (is.null(units)) {
+        units <- seq_len(x$nUnit)
+    }
     if (!is.null(names)) stopifnot(length(units) == length(names))
-    
+    if (isTRUE(decompose)) decompose <- colnames(x$stsObj)
+
+    ## get decomposed mean => no need to compute it in each plotHHH4_fitted1()
+    if (is.null(meanHHH)) {
+        meanHHH <- if (is.null(decompose)) {
+            meanHHH(x$coefficients, terms.hhh4(x))
+        } else {
+            decompose.hhh4(x)
+        }
+    }
+
+    ## check color vector
     col <- if (is.null(decompose) && length(col) == 4) {
         ## compatibility with surveillance < 1.10-0
         pt.col <- col[4L]
         rev(col[-4L])
     } else {
-        if (isTRUE(decompose)) decompose <- colnames(x$stsObj)
-        plotHHH4_fitted_check_col_decompose(col, decompose, colnames(x$stsObj))
+        plotHHH4_fitted_check_col_decompose(col, decompose)
     }
-    
+
+    ## setup graphical parameters
     if (is.list(par.settings)) {
         par.defaults <- list(mfrow = sort(n2mfrow(length(units))),
                              mar = c(4,4,2,0.5)+.1, las = 1)
@@ -85,35 +100,24 @@ plotHHH4_fitted <- function (x, units = 1, names = NULL,
         legend.args <- modifyList(default.args, legend.args)
     }
 
-    ## get decomposed mean
-    if (is.null(meanHHH)) {
-        meanHHH <- if (is.null(decompose)) {
-            meanHHH(x$coefficients, terms.hhh4(x))
-        } else {
-            decompose.hhh4(x)
-        }
-    }
-    
     ## plot fitted values region by region
     meanHHHunits <- vector(mode="list", length=length(units))
     names(meanHHHunits) <- if (is.character(units)) units else colnames(x$stsObj)[units]
     for(i in seq_along(units)) {
-        meanHHHunits[[i]] <- plotHHH4_fitted1(x, units[i], main=names[i],
+        meanHHHunits[[i]] <- plotHHH4_fitted1(x, unit=units[i], main=names[i],
                                               col=col, pch=pch, pt.cex=pt.cex, pt.col=pt.col,
-                                              decompose=decompose, meanHHH=meanHHH, ...)
+                                              decompose=decompose, total=total, meanHHH=meanHHH, ...)
         if (i %in% legend) do.call("legend", args=legend.args)
     }
     invisible(meanHHHunits)
 }
 
-plotHHH4_fitted_check_col_decompose <- function (col, decompose, unitNames)
+plotHHH4_fitted_check_col_decompose <- function (col, decompose)
 {
     if (is.null(decompose)) {
-        stopifnot(length(col) == 3)
+        stopifnot(length(col) == 3L)
     } else {
-        nUnit <- length(unitNames)
-        stopifnot(length(decompose) == nUnit,
-                  setequal(decompose, unitNames))
+        nUnit <- length(decompose)
         if (length(col) == nUnit) {
             col <- c("grey85", col)  # first color is for "endemic"
         } else if (length(col) != 1L + nUnit) {
@@ -132,25 +136,19 @@ plotHHH4_fitted1 <- function(x, unit=1, main=NULL,
                              pch=19, pt.cex=0.6, pt.col=1, border=col,
                              start=x$stsObj@start, end=NULL, xaxis=NULL,
                              xlim=NULL, ylim=NULL, xlab="", ylab="No. infected",
-                             hide0s=FALSE, decompose=NULL, meanHHH=NULL)
+                             hide0s=FALSE, decompose=NULL, total=FALSE, meanHHH=NULL)
 {
     stsObj <- x$stsObj
-    if (is.character(unit) &&
+    if (!total && is.character(unit) &&
         is.na(unit <- match(.unit <- unit, colnames(stsObj))))
         stop("region '", .unit, "' does not exist")
-    if (is.null(main)) main <- colnames(stsObj)[unit]
-
-    col <- if (is.null(decompose) && length(col) == 4) {
-        ## compatibility with surveillance < 1.10-0
-        pt.col <- col[4L]
-        rev(col[-4L])
-    } else {
-        if (isTRUE(decompose)) decompose <- colnames(x$stsObj)
-        plotHHH4_fitted_check_col_decompose(col, decompose, colnames(x$stsObj))
-    }
+    if (is.null(main))
+        main <- if (total) "Overall" else colnames(stsObj)[unit]
+    if (isTRUE(decompose))
+        decompose <- colnames(stsObj)
 
     ## get observed counts
-    obs <- observed(stsObj)[,unit]
+    obs <- if (total) rowSums(observed(stsObj)) else observed(stsObj)[,unit]
 
     ## time range for plotting
     start0 <- yearepoch2point(stsObj@start, stsObj@freq, toleft=TRUE)
@@ -171,14 +169,27 @@ plotHHH4_fitted1 <- function(x, unit=1, main=NULL,
     }
 
     ## get fitted component means
+    if (is.null(meanHHH)) {
+        meanHHH <- if (is.null(decompose)) {
+            meanHHH(x$coefficients, terms.hhh4(x))
+        } else {
+            decompose.hhh4(x)
+        }
+    }
     meanHHHunit <- if (is.null(decompose)) {
-        if (is.null(meanHHH))
-            meanHHH <- meanHHH(x$coefficients, terms.hhh4(x))
-        sapply(meanHHH, "[", i=TRUE, j=unit)
+        if (total) {
+            sapply(meanHHH, rowSums)
+        } else {
+            sapply(meanHHH, "[", i=TRUE, j=unit)
+        }
     } else {
-        if (is.null(meanHHH))
-            meanHHH <- decompose.hhh4(x)
-        meanHHH[,unit,c("endemic",decompose)]
+        if (!setequal(decompose, dimnames(meanHHH)[[3L]][-1L]))
+            stop("'decompose' must be (a permutation of) the fitted units")
+        if (total) {
+            apply(meanHHH[,,c("endemic",decompose)], c(1L, 3L), sum)
+        } else {
+            meanHHH[,unit,c("endemic",decompose)]
+        }
     }
     stopifnot(is.matrix(meanHHHunit), !is.null(colnames(meanHHHunit)),
               nrow(meanHHHunit) == length(x$control$subset))
@@ -188,7 +199,16 @@ plotHHH4_fitted1 <- function(x, unit=1, main=NULL,
         ## a workaround is then to set 'start' to a later time point
         stop("predicted mean contains missing values")
     }
-    
+
+    ## check color vector
+    col <- if (is.null(decompose) && length(col) == 4L) {
+        ## compatibility with surveillance < 1.10-0
+        pt.col <- col[4L]
+        rev(col[-4L])
+    } else {
+        plotHHH4_fitted_check_col_decompose(col, decompose)
+    }
+
     ## establish basic plot window
     if (is.null(ylim)) ylim <- c(0, max(obs[tpInRange],na.rm=TRUE))
     plot(c(start,end), ylim, xlim=xlim, xlab=xlab, ylab=ylab, type="n",
@@ -208,7 +228,7 @@ plotHHH4_fitted1 <- function(x, unit=1, main=NULL,
         plotComponentPolygons(x = tp[tpInSubset], y = meanHHHunit[, non0, drop = FALSE],
                               col = col[non0], border = border[non0], add = TRUE)
     }
-    
+
     ## add observed counts within [start;end]
     ptidx <- if (hide0s) intersect(tpInRange, which(obs > 0)) else tpInRange
     points(tp[ptidx], obs[ptidx], col=pt.col, pch=pch, cex=pt.cex)
@@ -228,16 +248,16 @@ plotComponentPolygons <- function (x, y, col = 1:6, border = col, add = FALSE)
     yc <- if ((nPoly <- ncol(y)) > 1L) {
         apply(X = y, MARGIN = 1L, FUN = cumsum) # nPoly x nTime
     } else t(y)
-    
+
     if (!add) {
         ## establish basic plot window
         plot(range(x), range(yc[nPoly,]), type = "n")
     }
-    
+
     ## recycle graphical parameters
     col <- rep_len(col, nPoly)
     border <- rep_len(border, nPoly)
-    
+
     ## draw polygons
     xpoly <- c(x[1L], x, x[length(x)])
     for (poly in nPoly:1) {
@@ -259,34 +279,34 @@ plotHHH4_maps <- function (x,
     map = x$stsObj@map, meanHHH = NULL)
 {
     which <- match.arg(which, several.ok = TRUE)
-    
+
     ## extract district-specific mean components
     if (is.null(meanHHH)) {
         meanHHH <- meanHHH(x$coefficients, terms.hhh4(x))
     }
-    
+
     ## select relevant components and convert to an array
     meanHHH <- simplify2array(
         meanHHH[c("mean", "endemic", "epi.own", "epi.neighbours")],
         higher = TRUE)
-    
+
     ## convert to proportions
     if (prop) {
         meanHHH[,,-1L] <- meanHHH[,,-1L,drop=FALSE] / c(meanHHH[,,1L])
     }
-    
+
     ## select only 'which' components
     meanHHH <- meanHHH[,,which,drop=FALSE]
-    
+
     ## check map
     map <- as(map, "SpatialPolygonsDataFrame")
     if (!all(dimnames(meanHHH)[[2L]] %in% row.names(map))) {
         stop("'row.names(map)' do not cover all fitted districts")
     }
-    
+
     ## average over time
     comps <- as.data.frame(colMeans(meanHHH, dims = 1))
-    
+
     ## attach to map data
     map@data <- cbind(map@data, comps[row.names(map),,drop=FALSE])
 
@@ -304,7 +324,7 @@ plotHHH4_maps <- function (x,
     if (!is.null(layout.labels <- layout.labels(map, labels))) {
         sp.layout <- c(sp.layout, list(layout.labels))
     }
-    
+
     ## produce maps
     grobs <- mapply(
         FUN = function (zcol, main, zmax)
@@ -330,26 +350,26 @@ plotHHH4_ri <- function (x, component, labels = FALSE, sp.layout = NULL,
                          gpar.missing = list(col="darkgrey", lty=2, lwd=2),
                          ...)
 {
-    ranefmatrix <- ranef(x, tomatrix=TRUE)
+    ranefmatrix <- ranef.hhh4(x, tomatrix=TRUE)
     if (is.null(ranefmatrix)) stop("model has no random effects")
     stopifnot(length(component) == 1L)
     if (is.na(comp <- pmatch(component, colnames(ranefmatrix))))
         stop("'component' must (partially) match one of ",
              paste(dQuote(colnames(ranefmatrix)), collapse=", "))
-    
+
     map <- as(x$stsObj@map, "SpatialPolygonsDataFrame")
     if (length(map) == 0L) stop("'x$stsObj' has no map")
     map$ranef <- ranefmatrix[,comp][row.names(map)]
-    
+
     if (is.list(gpar.missing) && any(is.na(map$ranef))) {
-        sp.layout <- c(sp.layout, 
+        sp.layout <- c(sp.layout,
                        c(list("sp.polygons", map[is.na(map$ranef),]),
                          gpar.missing))
     }
     if (!is.null(layout.labels <- layout.labels(map, labels))) {
         sp.layout <- c(sp.layout, list(layout.labels))
     }
-    
+
     spplot(map[!is.na(map$ranef),], zcol = "ranef",
            sp.layout = sp.layout, ...)
 }
@@ -431,10 +451,10 @@ createLambda <- function (object)
         attr(Lambda, "type") <- "zero"
         return(Lambda)
     }
-    
+
     meanHHH <- meanHHH(object$coefficients, terms(object),
                        subset=seq_len(nTime))
-    
+
     W <- getNEweights(object)
     Wt <- if (is.null(W)) {
         NULL
@@ -454,7 +474,7 @@ createLambda <- function (object)
     } else {
         function (t) {
             stopifnot(isScalar(t) && t > 0 && t <= nTime)
-            Lambda <- meanHHH$ne.exppred[t,] * Wt(t)
+            Lambda <- meanHHH$ne.exppred[t,] * t(Wt(t))
             diag(Lambda) <- diag(Lambda) + meanHHH$ar.exppred[t,]
             Lambda
         }
@@ -473,7 +493,7 @@ maxEV <- function (Lambda,
     } else {
         eigen(Lambda, symmetric = symmetric, only.values = TRUE)$values[1L]
     }
-    
+
     ## dominant eigenvalue may be complex
     if (is.complex(maxEV)) {
         if (Im(maxEV) == 0) { # if other eigenvalues are complex
@@ -533,8 +553,8 @@ plotHHH4_season <- function (...,
             setNames(rep(list(x), length(defaults)), names(defaults))
         } else stop("'", deparse(substitute(x)), "' is not suitably specified")
     }
-    
-    ## component-specific arguments 
+
+    ## component-specific arguments
     ylim <- withDefaults(ylim,
                          list(ar=NULL, ne=NULL, end=NULL, maxEV=NULL))
     ylab <- withDefaults(ylab,
@@ -544,7 +564,7 @@ plotHHH4_season <- function (...,
                               maxEV="dominant eigenvalue"))
     main <- withDefaults(main,
                          list(ar="autoregressive component",
-                              ne="spatiotemporal component",    
+                              ne="spatiotemporal component",
                               end="endemic component",
                               maxEV="dominant eigenvalue"))
     anyMain <- any(unlist(lapply(main, nchar),
@@ -594,6 +614,9 @@ plotHHH4_season <- function (...,
                 c(list(seasons[[comp]], xlim=xlim, ylim=ylim[[comp]],
                        xlab=xlab, ylab=ylab[[comp]], main=main[[comp]]),
                   matplot.args))
+        if (is.list(refline.args) && !intercept && any(seasons[[comp]] != 1))
+            do.call("abline", modifyList(list(h=1, lty=3, col="grey"),
+                                         refline.args))
         if (match(comp, components) %in% legend)
             do.call("legend", legend.args)
     }
@@ -626,18 +649,18 @@ plotHHH4_season <- function (...,
 #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 getSeason <- function(x, component = c("end", "ar", "ne"), unit = 1)
 {
-    stopifnot(inherits(x, c("hhh4","ah4")))
+    stopifnot(inherits(x, "hhh4"))
     component <- match.arg(component)
     startseason <- getSeasonStart(x)
     freq <- x$stsObj@freq
     if (is.character(unit)) unit <- match(unit, colnames(x$stsObj))
-    
+
     ## return -Inf is component is not in the model (-> exp(-Inf) = 0)
     if (!component %in% componentsHHH4(x))
         return(list(intercept=-Inf, season=rep.int(-Inf, freq)))
 
     ## get the intercept
-    est <- fixef(x, reparamPsi=FALSE)
+    est <- fixef.hhh4(x, reparamPsi=FALSE)
     intercept <- unname(est[grep(paste0("^", component, "\\.(1|ri)"), names(est))])
     if (length(intercept) == 0) {
         intercept <- 0 # no intercept (not standard)
@@ -677,7 +700,7 @@ getSeason <- function(x, component = c("end", "ar", "ne"), unit = 1)
 #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 getMaxEV_season <- function (x)
 {
-    stopifnot(inherits(x, c("hhh4","ah4")))
+    stopifnot(inherits(x, "hhh4"))
     nUnits <- x$nUnit
     freq <- x$stsObj@freq
     components <- componentsHHH4(x)
@@ -697,13 +720,13 @@ getMaxEV_season <- function (x)
                     "covariates/offsets not accounted for;\n",
                     "  use getMaxEV() or plotHHH4_maxEV()")
     }
-    
+
     ## global intercepts and seasonality
     s2.lambda <- getSeason(x, "ar")
     s2.phi <- getSeason(x, "ne")
-    
+
     ## unit-specific intercepts
-    ris <- ranef(x, tomatrix=TRUE)
+    ris <- ranef.hhh4(x, tomatrix=TRUE)
     ri.lambda <- ris[,pmatch("ar.ri", colnames(ris), nomatch=0L),drop=TRUE]
     if (length(ri.lambda) == 0L) ri.lambda <- rep.int(0, nUnits)
     ri.phi <- ris[,pmatch("ne.ri", colnames(ris), nomatch=0L),drop=TRUE]
@@ -720,7 +743,7 @@ getMaxEV_season <- function (x)
     createLambda <- function (t)
     {
         Lambda <- if ("ne" %in% components) {
-            exp(s2.phi$intercept + ri.phi + if(t==0) 0 else s2.phi$season[t]) * W
+            exp(s2.phi$intercept + ri.phi + if(t==0) 0 else s2.phi$season[t]) * t(W)
         } else matrix(0, nUnits, nUnits)
         if ("ar" %in% components) {
             diag(Lambda) <- diag(Lambda) + exp(s2.lambda$intercept + ri.lambda +
@@ -773,7 +796,7 @@ plotHHH4_neweights <- function (x, plotter = boxplot, ...,
                                 exclude = 0, maxlag = Inf)
 {
     plotter <- match.fun(plotter)
-    
+
     ## orders of neighbourhood (o_ji)
     nbmat <- neighbourhood(x$stsObj)
     if (all(nbmat %in% 0:1)) {
@@ -811,7 +834,7 @@ getHHH4list <- function (..., .names = NA_character_)
 {
     objects <- list(...)
     if (length(objects) == 1L && is.list(objects[[1L]]) &&
-        inherits(objects[[1L]][[1L]], c("hhh4","ah4"))) {
+        inherits(objects[[1L]][[1L]], "hhh4")) {
         ## ... is a single list of fits
         objects <- objects[[1L]]
         if (is.null(names(objects))) names(objects) <- seq_along(objects)
@@ -820,7 +843,7 @@ getHHH4list <- function (..., .names = NA_character_)
             ifelse(nzchar(names(objects)), names(objects), .names)
         }
     }
-    if (!all(sapply(objects, inherits, what=c("hhh4","ah4"))))
+    if (!all(sapply(objects, inherits, what="hhh4")))
         stop("'...' must consist of hhh4()-fits only")
 
     ## check common epoch, start and frequency and append them as attributes
@@ -836,7 +859,7 @@ getHHH4list <- function (..., .names = NA_character_)
     if (length(freq)>1)
         stop("supplied hhh4-models obey different frequencies")
     attr(objects, "freq") <- freq
-    
+
     ## done
     return(objects)
 }
