@@ -1,7 +1,7 @@
 ################################################################################
 ### Initialization and other basic methods for the S4 class "sts"
 ###
-### Copyright (C) 2007-2014 Michael Hoehle, 2012-2016 Sebastian Meyer
+### Copyright (C) 2007-2014 Michael Hoehle, 2012-2017 Sebastian Meyer
 ###
 ### This file is part of the R package "surveillance",
 ### free software under the terms of the GNU General Public License, version 2,
@@ -41,7 +41,7 @@ sts <- function (observed,
             warning("'population' takes precedence over 'populationFrac'")
         slots$populationFrac <- population
     } # else "populationFrac" is a possible element of ...
-    
+
     ## call the standard generator function with explicitly set slots
     isNULL <- vapply(X = slots, FUN = is.null,
                      FUN.VALUE = FALSE, USE.NAMES = FALSE)
@@ -74,7 +74,7 @@ init.sts <- function(.Object, ..., # also for slots of classes extending "sts"
 {
     if (nargs() < 2) # nothing to do
         return(.Object)
-    
+
     if (missing(observed)) { # use default initialize-method
         ## such that, e.g., initialize(stsObj, map=newMap) will set a new map
         ## and copy other slots from stsObj instead of (re-)setting to defaults,
@@ -89,7 +89,7 @@ init.sts <- function(.Object, ..., # also for slots of classes extending "sts"
         .Object <- fix.dimnames(.Object)
         return(.Object)
     }
-    
+
     ## Ensure matrix form (auto-conversion is useful for single time series)
     observed <- as.matrix(observed)
     nUnit <- ncol(observed)
@@ -97,13 +97,13 @@ init.sts <- function(.Object, ..., # also for slots of classes extending "sts"
     state <- as.matrix(state)
     alarm <- as.matrix(alarm)
     upperbound <- as.matrix(upperbound)
-    
+
     ## clear rownames and set colnames for the matrix of observed counts
     if (is.null(namesObs <- colnames(observed))){
         namesObs <- paste0("observed", seq_len(nUnit))
     }
     dimnames(observed) <- list(NULL, namesObs)
-    
+
     ## if there is only one state-vector for more than one area, repeat it
     if (nUnit > 1 && ncol(state) == 1 && length(state) == nTime) {
         state <- rep.int(state, nUnit)
@@ -120,7 +120,7 @@ init.sts <- function(.Object, ..., # also for slots of classes extending "sts"
     ## next would coerce a "SpatialPolygonsDataFrame" to "SpatialPolygons"
     if (!missing(map))
         .Object@map <- map
-    
+
     ## set all other slots (including for classes extending this class)
     ## using the default initialize-method
     .Object <- callNextMethod(.Object, ...,
@@ -129,10 +129,10 @@ init.sts <- function(.Object, ..., # also for slots of classes extending "sts"
                               neighbourhood=neighbourhood,
                               populationFrac=populationFrac)
     ## this also checks validObject(.Object)
-    
+
     ## make sure all arrays have the same dimnames
     .Object <- fix.dimnames(.Object)
-    
+
     return(.Object)
 }
 
@@ -174,7 +174,7 @@ sts2disProg <- function(sts) {
 #The other alternative is to aggregate all units.
 #
 # Note: The function is not 100% consistent with what the generic
-#       aggregate does. 
+#       aggregate does.
 #
 # Warning: In case the aggregation is by unit the upperbound slot is set
 #          to NA. Furthermore the MAP object is left as.is, but
@@ -187,9 +187,13 @@ sts2disProg <- function(sts) {
 ###########################################################################
 
 setMethod("aggregate", signature(x="sts"), function(x,by="time",nfreq="all",...) {
-  
- #Action of aggregation for populationFrac depends on the type 
- binaryTS <- sum( x@populationFrac > 1 ) > 1  # FIXME: x@multinomialTS?
+
+ by <- match.arg(by, choices = c("time", "unit"))
+
+ ## Action of aggregation for populationFrac depends on the type
+ binaryTS <- sum( x@populationFrac > 1 ) > 1  # FIXME @ Michael: why not any()?
+ ## NOTE: we cannot rely on x@multinomialTS since this is not necessarily set
+ ##       if population(x) contains absolute numbers
 
   #Aggregate time
   if (by == "time") {
@@ -202,19 +206,19 @@ setMethod("aggregate", signature(x="sts"), function(x,by="time",nfreq="all",...)
       if (howmany - ceiling(howmany) != 0)
           stop("nfreq has to be a multiple of x@freq.")
     }
-    
+
     n <- dim(x@observed)[1]
     m <- ceiling(n/howmany)
     new <- rep(1:m,each=howmany)[1:n]
     x@freq <- ifelse(nfreq == "all", howmany, nfreq)
     x@epoch <- 1:m
-    
+
     x@observed <- as.matrix(aggregate(x@observed,by=list(new),sum)[,-1])
     x@state <- as.matrix(aggregate(x@state,by=list(new),sum)[,-1])>0
     x@alarm <- as.matrix(aggregate(x@alarm,by=list(new),sum)[,-1]) # number of alarms
     x@upperbound <- as.matrix(aggregate(x@upperbound,by=list(new),sum)[,-1])
     x@populationFrac <- as.matrix(aggregate(x@populationFrac,by=list(new),sum)[,-1])
-    ## FIXME: should make clear (warn?) that population is summed over time
+    ## CAVE: summing population (fractions) over time might not be intended
 
     #the population fractions need to be recomputed if not a binary ts
     if (!binaryTS) {
@@ -222,6 +226,8 @@ setMethod("aggregate", signature(x="sts"), function(x,by="time",nfreq="all",...)
       x@populationFrac <-x@populationFrac/sums
     }
   }
+
+  #Aggregate units
   if (by == "unit") {
     #Aggregate units
     x@observed <- as.matrix(apply(x@observed, MARGIN=1, sum))
@@ -231,14 +237,18 @@ setMethod("aggregate", signature(x="sts"), function(x,by="time",nfreq="all",...)
     x@upperbound <- matrix(NA_real_,ncol=ncol(x@alarm),nrow=nrow(x@alarm))
     x@populationFrac <- as.matrix(apply(x@populationFrac, MARGIN=1, sum))#>0
     x@neighbourhood <- matrix(NA, 1, 1) # consistent with default for new("sts")
-    ## FIXME: x@map will be invalid, remove or unionSpatialPolygons()?
+    ## we have lost colnames
+    colnames(x@observed) <- "overall"
+    x <- fix.dimnames(x)
+    ## drop the map (set to empty prototype)
+    x@map <- new(getSlots("sts")[["map"]])
   }
 
   #validObject(x) #just a check
 
   return(x)
 })
-  
+
 
 #####################################################################
 # Miscellaneous access methods
@@ -264,7 +274,7 @@ setMethod("year", "sts", function(x,...) {
   if (x@epochAsDate) {
     return(as.numeric(formatDate(epoch(x),"%G")))
   } else {
-    ((x@epoch-1 + x@start[2]-1) + (x@freq*x@start[1])) %/% x@freq 
+    ((x@epoch-1 + x@start[2]-1) + (x@freq*x@start[1])) %/% x@freq
   }
 })
 
@@ -285,10 +295,10 @@ setMethod("[", "sts", function(x, i, j, ..., drop) {
 
   x@populationFrac <- x@populationFrac[i,j,drop=FALSE]
   #If not binary TS the populationFrac is normed
-  binaryTS <- sum( x@populationFrac > 1 ) > 1 # FIXME @ Michael: x@multinomialTS
+  binaryTS <- sum( x@populationFrac > 1 ) > 1 # FIXME @ Michael: why not any()?
   if (!binaryTS) {
     x@populationFrac <- x@populationFrac / apply(x@populationFrac,MARGIN=1,sum)
-   }  
+   }
   x@upperbound <- x@upperbound[i,j,drop=FALSE]
 
   #Neighbourhood matrix
@@ -309,16 +319,16 @@ setMethod("[", "sts", function(x, i, j, ..., drop) {
   }
   start <- x@start
   new.sampleNo <- start[2] + i.min - 1
-  start.year <- start[1] + (new.sampleNo - 1) %/% x@freq 
+  start.year <- start[1] + (new.sampleNo - 1) %/% x@freq
   start.sampleNo <- (new.sampleNo - 1) %% x@freq + 1
   x@start <- c(start.year,start.sampleNo)
   ## If !epochAsDate, we also have to update epoch since it is relative to start
   if (!x@epochAsDate) x@epoch <- x@epoch - i.min + 1
-                                                     
+
   ## Note: We do not automatically subset the map according to j, since
   ##       identical(row.names(map), colnames(observed))
   ##       is not a property of the sts-class; Unmonitored regions are allowed.
-  
+
   #Done
   return(x)
 })
@@ -327,7 +337,7 @@ setMethod("[", "sts", function(x, i, j, ..., drop) {
 #########################################################################
 ## Plot method ... the type argument specifies what type of plot to make
 ##
-## plot as multivariate time series:  type = observed ~ time | unit 
+## plot as multivariate time series:  type = observed ~ time | unit
 ## plot as map object aggregated over time: type = observed ~ 1 | unit
 ## new map implementation via: type = observed ~ unit
 ## the specific plot functions are in separate files (stsplot_*.R)
@@ -335,7 +345,7 @@ setMethod("[", "sts", function(x, i, j, ..., drop) {
 
 setMethod("plot", signature(x="sts", y="missing"),
           function (x, type = observed ~ time | unit, ...) {
-  
+
   # catch new implementation of time-aggregate map plot
   if (isTRUE(all.equal(observed ~ unit, type)))
       return(stsplot_space(x, ...))
@@ -412,3 +422,5 @@ setMethod( "show", "sts", function( object ){
       print( head(object@neighbourhood,n))
   }
 } )
+
+
