@@ -6,9 +6,9 @@
 ### Maximum Likelihood inference for the two-component spatio-temporal intensity
 ### model described in Meyer et al (2012), DOI: 10.1111/j.1541-0420.2011.01684.x
 ###
-### Copyright (C) 2009-2015 Sebastian Meyer
-### $Revision: 1321 $
-### $Date: 2015-04-27 23:19:13 +0200 (Mon, 27. Apr 2015) $
+### Copyright (C) 2009-2017 Sebastian Meyer
+### $Revision: 1983 $
+### $Date: 2017-10-06 09:04:54 +0200 (Fri, 06. Oct 2017) $
 ################################################################################
 
 
@@ -42,8 +42,8 @@ twinstim <- function (
 
     ## Clean the model environment when exiting the function
     on.exit(suppressWarnings(rm(cl, cumCIF, cumCIF.pb, data, doHessian,
-        eventDists, eventsData, finetune, neghess, fisherinfo, fit, fixed,
-        functions, globalEndemicIntercept, h.Intercept, inmfe, initpars,
+        eventsData, finetune, neghess, fisherinfo, fit, fixed,
+        functions, globalEndemicIntercept, inmfe, initpars,
         ll, negll, loglik, msgConvergence, msgNotConverged,
         mfe, mfhEvents, mfhGrid, model, my.na.action, na.action, namesOptimUser,
         namesOptimArgs, nlminbControl, nlminbRes, nlmObjective, nlmControl,
@@ -51,9 +51,9 @@ twinstim <- function (
         optimMethod, optimRes, optimRes1, optimValid,
         origenv.endemic, origenv.epidemic, partial,
         partialloglik, ptm, qmatrix, res, negsc, score, start, subset, tmpexpr,
-        typeSpecificEndemicIntercept, useScore, verbose, whichfixed, 
+        typeSpecificEndemicIntercept, useScore, verbose, whichfixed,
         inherits = FALSE)))
-    
+
     ## also set fixed[st]iafpars to FALSE (for free posteriori evaluations, and
     ## to be defined for score function evaluation with optim.args=NULL)
     on.exit(fixedsiafpars <- fixedtiafpars <- FALSE, add = TRUE)
@@ -161,7 +161,7 @@ twinstim <- function (
                        eps.t = eps.t, eps.s = eps.s, BLOCK = BLOCK,
                        obsInfLength = .obsInfLength, bdist = .bdist)
 
-    
+
     ### Extract essential information from model frame
 
     # 'inmfe' indexes rows of data$events@data and is necessary for subsetting
@@ -205,9 +205,6 @@ twinstim <- function (
         gIntUpper <- mfe[["(obsInfLength)"]]
         gIntLower <- pmax(0, t0-eventTimes)
         eventCoords <- coordinates(data$events)[inmfe,,drop=FALSE]
-        ## if (verbose && N > 6000)
-        ##     cat("calculating distance matrix of", N, "events ...\n")
-        eventDists <- as.matrix(dist(eventCoords, method = "euclidean"))
         influenceRegion <- data$events@data$.influenceRegion[inmfe]
         iRareas <- vapply(X = influenceRegion, FUN = attr, which = "area",
                           FUN.VALUE = 0, USE.NAMES = FALSE)
@@ -215,9 +212,9 @@ twinstim <- function (
             data$events@data$.sources
         } else { # re-determine because subsetting has invalidated row indexes
             if (verbose) cat("updating list of potential sources ...\n")
-            lapply(seq_len(N), function (i)
-                determineSources(i, eventTimes, removalTimes, eventDists[i,],
-                                 eps.s, eventTypes, qmatrix))
+            determineSources(eventTimes = eventTimes, eps.t = eps.t,
+                             eventCoords = eventCoords, eps.s = eps.s,
+                             eventTypes = eventTypes, qmatrix = qmatrix)
         }
         ## calculate sum_{k=1}^K q_{kappa_j,k} for all j = 1:N
         qSum <- unname(rowSums(qmatrix)[eventTypes])   # N-vector
@@ -227,7 +224,7 @@ twinstim <- function (
 
 
     ### Drop "terms" and restore original formula environment
-    
+
     epidemic <- formula(epidemic)
     if (epilink != "log") # set as attribute only if non-standard link function
         attr(epidemic, "link") <- epilink
@@ -261,7 +258,7 @@ twinstim <- function (
     ## (will be handled separately)
     typeSpecificEndemicIntercept <- "1 | type" %in% attr(endemic, "term.labels")
     if (typeSpecificEndemicIntercept) {
-        endemic <- update(endemic, ~ . - (1|type)) # this drops the terms attributes
+        endemic <- update.formula(endemic, ~ . - (1|type)) # this drops the terms attributes
         endemic <- terms(endemic, data = data$stgrid, keep.order = TRUE)
     }
 
@@ -304,7 +301,7 @@ twinstim <- function (
                                # for debugging & necessary for intensityplots
         gridBlocks <- mfhGrid[["(BLOCK)"]]
         histIntervals <- data$stgrid[!duplicated.default(
-            data$stgrid$BLOCK, nmax = gridBlocks[length(gridBlocks)]
+            data$stgrid$BLOCK, nmax = data$stgrid$BLOCK[length(data$stgrid$BLOCK)]
         ), c("BLOCK", "start", "stop")] # sorted
         row.names(histIntervals) <- NULL
         histIntervals <- histIntervals[histIntervals$start >= t0 &
@@ -345,9 +342,9 @@ twinstim <- function (
         hEventsExpr <- call("exp", hEventsExpr)
     } else if (verbose) message("no endemic component in model")
 
-    
+
     ### Drop "terms" and restore original formula environment
-    
+
     endemic <- if (typeSpecificEndemicIntercept) {
         ## re-add it to the endemic formula
         update.formula(formula(endemic), ~ (1|type) + .)
@@ -359,10 +356,16 @@ twinstim <- function (
     ## Alternatively, we could set it to parent.frame().
 
 
-    ### Check that there is at least one parameter
+    ### Stop if model is degenerate
 
-    if (!hash && !hase) {
-        stop("nothing to do: neither endemic nor epidemic parts were specified")
+    if (!hash) {
+        if (hase) {
+            if (nEventsWithoutSources <- sum(lengths(eventSources[includes]) == 0))
+                stop("found ", nEventsWithoutSources, " events without .sources ",
+                     "(impossible in a purely epidemic model)")
+        } else {
+            stop("nothing to do: neither endemic nor epidemic parts were specified")
+        }
     }
 
 
@@ -400,11 +403,12 @@ twinstim <- function (
         ## but do not parallelize for a memoised siaf.step (becomes slower)
         if (useParallel &&
             !is.null(attr(siaf, "knots")) && !is.null(attr(siaf, "maxRange")) &&
-            isTRUE(attr(environment(siaf$f)$ringAreas, "memoised"))) {
+            requireNamespace("memoise", quietly = TRUE) &&
+            memoise::is.memoised(environment(siaf$f)$ringAreas)) {
             cores <- 1L
             useParallel <- FALSE
         }
-        
+
         ## Define function that integrates the 'tiaf' function
         .tiafInt <- .tiafIntFUN()
 
@@ -437,7 +441,7 @@ twinstim <- function (
         .siafInt.args <- c(alist(siafpars), control.siaf$F)
 
     } else {
-        
+
         if (!missing(siaf) && !is.null(siaf))
             warning("'siaf' can only be modelled in conjunction with an 'epidemic' process")
         if (!missing(tiaf) && !is.null(tiaf))
@@ -871,7 +875,7 @@ twinstim <- function (
                 siafInt <- do.call("..siafInt", .siafInt.args) # N-vector
                 gs <- gammapred * siafInt # N-vector
                 sapply(includes, function (i) {
-                    timeSources <- determineSources(i, eventTimes, removalTimes,
+                    timeSources <- determineSources1(i, eventTimes, removalTimes,
                         0, Inf, NULL)
                     nSources <- length(timeSources)
                     if (nSources == 0L) 0 else {
@@ -905,14 +909,14 @@ twinstim <- function (
                       sc = if (useScore) score else NULL,
                       fi = if (useScore) fisherinfo else NULL)
 
-    
+
     ### Include check for validity of siafpars and tiafpars ('validpars') in ll
 
     if (!is.null(siaf$validpars)) {
         body(ll) <- as.call(append(as.list(body(ll)),
             as.list(expression(
                 if (hassiafpars && !siaf$validpars(siafpars)) {
-                    if (optimArgs$control$trace > 0L)
+                    if (!isTRUE(optimArgs$control$trace == 0)) # default: NULL
                         cat("(invalid 'siafpars' in loglik)\n")
                     return(-Inf)
                 }
@@ -924,7 +928,7 @@ twinstim <- function (
         body(ll) <- as.call(append(as.list(body(ll)),
             as.list(expression(
                 if (hastiafpars && !tiaf$validpars(tiafpars)) {
-                    if (optimArgs$control$trace > 0L)
+                    if (!isTRUE(optimArgs$control$trace == 0)) # default: NULL
                         cat("(invalid 'tiafpars' in loglik)\n")
                     return(-Inf)
                 }
@@ -956,15 +960,35 @@ twinstim <- function (
 
     ### Check initial value for theta
 
-    if (is.null(optim.args[["par"]])) { # set naive defaults
-        h.Intercept <- if (nbeta0 > 0) rep.int(crudebeta0(
+    initpars <- rep(0, npars)
+    names(initpars) <- c(
+        if (nbeta0 > 1L) {
+            paste0("h.type",typeNames)
+        } else if (nbeta0 == 1L) "h.(Intercept)",
+        if (p > 0L) paste("h", colnames(mmhEvents), sep = "."),
+        if (hase) paste("e", colnames(mme), sep = "."),
+        if (hassiafpars) paste("e.siaf", seq_len(nsiafpars), sep="."),
+        if (hastiafpars) paste("e.tiaf", seq_len(ntiafpars), sep=".")
+    )
+
+    ## some naive defaults
+    if (nbeta0 > 0)
+        initpars[seq_len(nbeta0)] <- crudebeta0(
             nEvents = Nin,
             offset.mean = if (is.null(offsetGrid)) 0 else weighted.mean(offsetGrid, ds),
             W.area = sum(ds[gridBlocks==histIntervals[1,"BLOCK"]]),
             period = T-t0, nTypes = nTypes
-            ), nbeta0) else numeric(0L)
-        optim.args$par <- c(h.Intercept, rep.int(0, npars - nbeta0))
-    } else { # check validity of par-specification
+        )
+    if (hase && "e.(Intercept)" %in% names(initpars) && epilink == "log")
+        initpars["e.(Intercept)"] <- -9  # suitable value depends on [st]iafInt
+    if (hassiafpars && identical(body(siaf$f)[[2L]], quote(sds <- exp(pars)))) {
+        ## "detect" siaf.gaussian => use 10% of bbox diameter as initial sd
+        initpars[paste0("e.siaf.", seq_len(nsiafpars))] <-
+            round(log(0.1*sqrt(sum(apply(bbox(data$W), 1L, diff.default)^2))))
+    }
+
+    ## manual par-specification overrides these defaults
+    if (!is.null(optim.args[["par"]])) {
         if (!is.vector(optim.args$par, mode="numeric")) {
             stop("'optim.args$par' must be a numeric vector")
         }
@@ -973,27 +997,23 @@ twinstim <- function (
                                 "length as the number of unknown parameters (%d)"),
                           length(optim.args$par), npars))
         }
+        initpars[] <- optim.args$par
     }
-    
-    ## Set names for theta
-    names(optim.args$par) <- c(
-        if (nbeta0 > 1L) {
-            paste0("h.type",typeNames)
-        } else if (nbeta0 == 1L) "h.(Intercept)",
-        if (p > 0L) paste("h", colnames(mmhEvents), sep = "."),
-        if (hase) paste("e", colnames(mme), sep = "."),
-        if (hassiafpars) paste("e.siaf",1:nsiafpars,sep="."),
-        if (hastiafpars) paste("e.tiaf",1:ntiafpars,sep=".")
-    )
 
-    ## values in "start" overwrite initial values given by optim.args$par
+    ## values in "start" overwrite defaults and optim.args$par
     if (!is.null(start)) {
         start <- check_twinstim_start(start)
-        start <- start[names(start) %in% names(optim.args$par)]
-        optim.args$par[names(start)] <- start
+        start <- start[names(start) %in% names(initpars)]
+        initpars[names(start)] <- start
     }
 
-    initpars <- optim.args$par
+    ## warn if initial intercept is negative when the identity link is used
+    if (epilink == "identity" && "e.(Intercept)" %in% names(initpars) &&
+        initpars["e.(Intercept)"] < 0)
+        warning("identity link and negative start value for \"e.(Intercept)\"")
+
+    ## update optim.args$par
+    optim.args$par <- initpars
 
 
     ### Fixed parameters during optimization
@@ -1027,7 +1047,7 @@ twinstim <- function (
 
     ### Define negative log-likelihood (score, hessian) for minimization
     ### as a function of the non-fixed parameters
-    
+
     negll <- ll
     body(negll)[[length(body(negll))]] <-
         call("-", body(negll)[[length(body(negll))]])
@@ -1064,7 +1084,7 @@ twinstim <- function (
                      quote(!fixed), quote(!fixed), drop=FALSE)
         }
 
-        ## if siafpars or tiafpars are fixed, pre-evaluate integrals    
+        ## if siafpars or tiafpars are fixed, pre-evaluate integrals
         if (fixedsiafpars) {
             if (verbose)
                 cat("pre-evaluating 'siaf' integrals with fixed parameters ...\n")
@@ -1074,8 +1094,10 @@ twinstim <- function (
                     "are redundant and slow!\n",
                     "         Really consider installing package \"memoise\"!\n",
                     sep="")
-            .siafInt.args[[1]] <- initpars[paste("e.siaf", 1:nsiafpars, sep=".")]
-            siafInt <- do.call("..siafInt", .siafInt.args) # memoise()d
+            siafInt <- local({
+                siafpars <- initpars[paste("e.siaf", 1:nsiafpars, sep=".")]
+                do.call("..siafInt", .siafInt.args) # memoise()d
+            })
         }
         if (fixedtiafpars) {
             if (verbose) cat("pre-evaluating 'tiaf' integrals with fixed parameters ...\n")
@@ -1218,11 +1240,11 @@ twinstim <- function (
 
 
         ## Convergence message
-        
+
         msgConvergence <- if (finetune || optimMethod != "nlminb") {
             paste("code", optimRes$convergence)
         } else optimRes$message
-        
+
         if (optimRes$convergence != 0) {
             msgNotConverged <- paste0("optimization routine did not converge (",
                                       msgConvergence, ")")
@@ -1287,7 +1309,7 @@ twinstim <- function (
             dimnames = list(names(initpars), names(initpars))
             )
         )
-    
+
     # If requested, add observed fisher info (= negative hessian at maximum)
     fit["fisherinfo.observed"] <- list(
         if (any(!fixed) && !is.null(optimRes$hessian)) optimRes$hessian
@@ -1312,7 +1334,7 @@ twinstim <- function (
         if (!fixedsiafpars) siafInt <- do.call("..siafInt", .siafInt.args)
         if (!fixedtiafpars) tiafInt <- .tiafInt(tiafpars)
     }
-    
+
     # fitted intensities
     hEvents <- if (hash) .hEvents(unname(beta0), beta) else rep.int(0, Nin)
     eEvents <- if (hase) {
@@ -1321,7 +1343,7 @@ twinstim <- function (
     fit$fitted <- hEvents + eEvents   # = lambdaEvents  # Nin-vector
     fit$fittedComponents <- cbind(h = hEvents, e = eEvents)
     rm(hEvents, eEvents)
-    
+
     # calculate cumulative ground intensities at event times
     # Note: this function is also used by residuals.twinstim
     LambdagEvents <- function (cores = 1L, cumCIF.pb = interactive())
@@ -1357,7 +1379,7 @@ twinstim <- function (
     fit$R0 <- if (hase) qSum * gammapred * siafInt * tiafInt else rep.int(0, N)
     names(fit$R0) <- row.names(mfe)
 
-    
+
     ### Append model information
 
     fit$npars <- c(nbeta0 = nbeta0, p = p,
@@ -1369,7 +1391,7 @@ twinstim <- function (
                         siaf = siaf, tiaf = tiaf)
     fit["control.siaf"] <- list(control.siaf)    # might be NULL
 
-    
+
     ### Append optimizer configuration
 
     optim.args$par <- initpars        # reset to also include fixed coefficients
