@@ -6,9 +6,9 @@
 ### Spatial interaction functions for twinstim's epidemic component.
 ### Specific implementations are in seperate files (e.g.: Gaussian, power law).
 ###
-### Copyright (C) 2009-2015 Sebastian Meyer
-### $Revision: 1217 $
-### $Date: 2015-03-07 02:30:09 +0100 (Sam, 07. Mär 2015) $
+### Copyright (C) 2009-2015,2017 Sebastian Meyer
+### $Revision: 1965 $
+### $Date: 2017-10-02 16:10:56 +0200 (Mon, 02. Oct 2017) $
 ################################################################################
 
 
@@ -111,12 +111,15 @@ siaf.constant <- function ()
 ##########################################
 
 ## numerical integration of f over a polygonal domain (single "owin" and type)
-siaf.fallback.F <- function(polydomain, f, pars, type, method = "SV", ...)
+siaf.fallback.F <- function (polydomain, f, pars, type, method = "SV", ...)
 {
-    if (identical(method,"SV"))
-        polyCub.SV(polydomain, f, pars, type, alpha=0, ...) # since max at origin
-    else 
-        polyCub(polydomain, f, method, pars, type, ...)
+    if (identical(method,"SV")) {
+        polyCub.SV(polyregion = polydomain, f = f, pars, type,
+                   alpha = 0, ...) # since max at origin
+    } else {
+        polyCub(polyregion = polydomain, f = f, method = method,
+                pars, type, ...)
+    }
 }
 
 ## numerical integration of f over a circular domain
@@ -138,8 +141,10 @@ siaf.fallback.Deriv <- function (polydomain, deriv, pars, type,
     deriv1 <- function (s, paridx)
         deriv(s, pars, type)[,paridx,drop=TRUE]
     intderiv1 <- function (paridx)
-        polyCub(polydomain, deriv1, method, paridx=paridx, ...)
-    sapply(seq_along(pars), intderiv1)
+        polyCub(polyregion = polydomain, f = deriv1, method = method,
+                paridx = paridx, ...)
+    vapply(X = seq_along(pars), FUN = intderiv1,
+           FUN.VALUE = 0, USE.NAMES = FALSE)
 }
 
 
@@ -176,7 +181,7 @@ siaf.simulatePC <- function (intrfr)    # e.g., intrfr.powerlaw
         r <- vapply(X=runif(n), FUN=QF, FUN.VALUE=0, USE.NAMES=FALSE)
         ## Check simulation of r via kernel estimate:
         ## plot(density(r, from=0, to=ub)); curve(p(x)/normconst,add=TRUE,col=2)
-        
+
         ## now rotate each point by a random angle to cover all directions
         theta <- runif(n, 0, 2*pi)
         r * cbind(cos(theta), sin(theta))
@@ -207,7 +212,7 @@ checksiaf <- function (siaf, pargrid, type = 1, tolerance = 1e-5,
                       check.attributes=FALSE, tolerance=tolerance),
             "\n")
     }
-    
+
     ## Check 'Fcircle'
     if (!is.null(siaf$Fcircle)) {
         cat("'Fcircle' vs. cubature using method = \"",method,"\" ... ", sep="")
@@ -218,7 +223,7 @@ checksiaf <- function (siaf, pargrid, type = 1, tolerance = 1e-5,
                       check.attributes=FALSE, tolerance=tolerance),
             "\n")
     }
-    
+
     ## Check 'deriv'
     if (!is.null(siaf$deriv)) {
         cat("'deriv' vs. numerical derivative ... ")
@@ -263,12 +268,10 @@ checksiaf <- function (siaf, pargrid, type = 1, tolerance = 1e-5,
 
 checksiaf.F <- function (F, f, pargrid, type=1, method="SV", ...)
 {
-    letterR <- "cheating on codetools::checkUsage"
-    data("letterR", package="spatstat", envir=environment())
-    poly <- shift.owin(letterR, -c(3,2))
     res <- t(apply(pargrid, 1, function (pars) {
-        given <- F(poly, f, pars, type)
-        num <- siaf.fallback.F(poly, f, pars, type, method=method, ...)
+        given <- F(LETTERR, f, pars, type)
+        num <- siaf.fallback.F(polydomain = LETTERR, f = f, pars = pars,
+                               type = type, method = method, ...)
         c(given, num)
     }))
     colnames(res) <- c("F", method)
@@ -281,9 +284,10 @@ checksiaf.Fcircle <- function (Fcircle, f, pargrid, type=1,
     pargrid <- pargrid[rep(1:nrow(pargrid), each=length(rs)),,drop=FALSE]
     rpargrid <- cbind(rs, pargrid, deparse.level=0)
     res <- t(apply(rpargrid, 1, function (x) {
-        c(ana = Fcircle(x[1], x[-1], type),
-          num = siaf.fallback.F(discpoly(c(0,0), x[1], npoly=128, class="owin"),
-                                f, x[-1], type, method=method, ...))
+        disc <- discpoly(c(0,0), x[1L], npoly = 128, class = "owin")
+        c(ana = Fcircle(x[1L], x[-1L], type),
+          num = siaf.fallback.F(polydomain = disc, f = f, pars = x[-1L],
+                                type = type, method = method, ...))
     }))
     res
 }
@@ -302,12 +306,10 @@ checksiaf.deriv <- function (deriv, f, pargrid, type=1, rmax=100)
 
 checksiaf.Deriv <- function (Deriv, deriv, pargrid, type=1, method="SV", ...)
 {
-    letterR <- "cheating on codetools::checkUsage"
-    data("letterR", package="spatstat", envir=environment())
-    poly <- shift.owin(letterR, -c(3,2))
     res <- t(apply(pargrid, 1, function (pars) {
-        given <- Deriv(poly, deriv, pars, type)
-        num <- siaf.fallback.Deriv(poly, deriv, pars, type, method=method, ...)
+        given <- Deriv(LETTERR, deriv, pars, type)
+        num <- siaf.fallback.Deriv(polydomain = LETTERR, deriv = deriv, pars = pars,
+                                   type = type, method = method, ...)
         c(given, num)
     }))
     paridxs <- seq_len(ncol(pargrid))
@@ -341,7 +343,7 @@ checksiaf.simulate <- function (simulate, f, pars, type=1, B=3000, ub=10,
         do.call("curve", list(quote(rfr(x)/rfrnorm), add=TRUE, col=2, lwd=2))
         ##<- use do.call-construct to prevent codetools::checkUsage from noting "x"
     }
-    
+
     ## invisibly return simulated points
     invisible(simpoints)
 }
